var searchQuery = "";
var searchHistory;

function loadSearchHistory(){
    searchHistory = new Array();
    var savedSearchHistory = preferences.searchHistory.option;
    for(var i = 0;i<savedSearchHistory;i++)
        searchHistory.push(savedSearchHistory[i]);
}

function initialize(){
    loadSearchHistory();
}

function cleanUp(){
    preferences.searchHistory.option = searchHistory;
}

function handleFailure(failureMsg) {
    resetDefinitionBox();
    updateStatusBar(failureMsg, true);
}

function doSearch() {
    var query = inputBox.data;
    if (query == "") return;
    fetchGooglebWebDefinitions(query);
}

function addToSearchHistory(query) {
    searchHistory.push(query);

    if (searchHistory.length > 5) {
        searchHistory.shift();
    }
}

function clearSearchHistory(){
    searchHistory = new Array();
}

function minimize(){
    changeVisibilityOfDefinitionBox(false);
}

function maximize(){
    changeVisibilityOfDefinitionBox(true);
}
function menuItemsFromSearchHistory() {
    var menuItems = new Array();

    if(searchHistory.length == 0) {
        var recentSearches = createMenuItem("No recent searches", false, "");
        menuItems.push(recentSearches);
        return menuItems;
    }

    var clearHistory = createMenuItem("Clear recent searches", true, "clearSearchHistory();");
    menuItems.push(clearHistory);

    var recentSearches = createMenuItem("Recent searches", false, "");
    menuItems.push(recentSearches);

    for (var i =0 ; i < searchHistory.length ; i++) {
        var menuItem = createMenuItem(searchHistory[i], true ,"");
        menuItems.push(menuItem);
    }
    return menuItems;
}


function createMenuItem(title, isEnabled, onSelect) {
    var item = new MenuItem();
    item.title = title;
    item.enabled = isEnabled;
    item.onSelect = onSelect;
    return item;
}

function buildContextMenu() {
    return buildStaticMenuItems().concat(menuItemsFromSearchHistory());
}

function buildStaticMenuItems() {

    var clearItem = createMenuItem("Clear search box", true, "resetWidget();");

    var isSearchQueryEmpty = (searchQuery == "");
    var onSelectViewDefinition= function(){
        var url = "http://www.google.com/search?q=" + "define:" + searchQuery;
        openURL(url);
    }
    var onSelectGoogleSearch= function(){
        var url = "http://www.google.com/search?q=" + searchQuery;
        openURL(url);
    }
    var viewDefinitionInBrowser = createMenuItem("View Google Glossary Definitions In Browser", !isSearchQueryEmpty , onSelectViewDefinition);
    var doGoogleSearchInBrowser = createMenuItem("View Google Web Search Results In Browser", !isSearchQueryEmpty , onSelectGoogleSearch);

    return new Array(clearItem,viewDefinitionInBrowser,doGoogleSearchInBrowser);
}


function resetDefinitionBox() {
    definitionBox.data = "";
    definitionBox.tooltip = "";
    definitionBox.onMouseUp = "";
    changeVisibilityOfDefinitionBox(false);
    error.visible = false;
}
function resetWidget() {
    resetInputBox();
    resetDefinitionBox();
    searchQuery = "";
}

function resetInputBox() {
    inputBox.data = "";
}

function showDefinitions(results) {
    changeVisibilityOfDefinitionBox(true);
    searchQuery = inputBox.data;
    addToSearchHistory(searchQuery);
    var definitions = formatResults(results);
    updateDefinitionBox(definitions);
}

function updateDefinitionBox(definitions) {
    definitionBox.data = unescape(definitions);
    definitionBox.tooltip = "Click here to view the definition in your web browser";
    definitionBox.onMouseUp = function() {
        var url = "http://www.google.com/search?q=" + "define:" + searchQuery;
        openURL(url);
    }
}

function formatResults(results) {
    var definitions = "";
    for (var i = 0; i < results.length; i++) {
        var prefix = "[ " + (i + 1) + " ]  ";
        var result = results[i];
        var suffix = "\n\n";
        definitions += prefix + result + suffix;
    }
    return definitions;
}

function updateStatusBar(message, isError) {
    changeVisibilityOfDefinitionBox(true);
    definitionBox.data = "   " + message;
    error.visible = isError;
}

function changeVisibilityOfDefinitionBox(isVisible) {
    definitionBoxFrame.visible = isVisible;
}

function extractDefinitions(googleResponse) {
    var grepString = "Definitions of <b>(.*?)</b> on the Web:";
    var matches = googleResponse.search(grepString);
    if (matches == -1)
        return "error";
    var matchedString = googleResponse.substring(matches, googleResponse.length);
    var startIndex = matchedString.indexOf("<li>") + "<li>".length;
    var endIndex = matchedString.indexOf("<br>");
    var defintion = matchedString.substring(startIndex, endIndex);
    var definitionArray = defintion.split("<li>");
    return definitionArray;
}

function fetchGooglebWebDefinitions(word) {
    resetDefinitionBox();
    updateStatusBar("Loading.....", false);
    sendRequestToGoogle(word);
}

function sendRequestToGoogle(query) {
    try {
        var url = "http://www.google.com/search?q=" + "define:" + query;
        var googleUrl = new URL();
        googleUrl.location = url;
        googleUrl.fetchAsync(callBackMethod);
    }
    catch(e) {
        handleFailure("Error while sending request to google");
    }
}

function callBackMethod(url) {
    var googleResponse = url.result;


    if (googleResponse.toLowerCase() == "could not load url") {
        handleFailure("Google server not available !");
        searchQuery ="";
        return;
    }


    var result = extractDefinitions(googleResponse);

    if (result == "error") {
        handleFailure("No definitions were found !");
        searchQuery ="";
        return;
    }
    showDefinitions(result);
}

function unescape(string) {
    var str = string;
    var specialChars = new Array();
    specialChars["&amp;"] = "&";
    specialChars["&quot;"] = "\"";
    specialChars["&apos:"] = "\'";
    specialChars["&lt"] = "<";
    specialChars["&gt"] = ">";
    str = str.replace(/&quot;/g, specialChars["&quot;"]);
    str = str.replace(/&amp;/g, specialChars["&amp;"]);
    str = str.replace(/&apos:/g, specialChars["&apos:"]);
    str = str.replace(/&gt/g, specialChars["&gt"]);
    str = str.replace(/&lt/g, specialChars["&lt"]);
    return str;
}
